/* pulsdsp.c
 */
/**
 * \page puls Pulsfrequenzmessung
 * \section dsp DSP Pulsdatenauswertung 
 *
 * Die von der Antenne kommenden Pulssignale des Brustgurs werden am Analogeingang mit 4*5.3 kHz gesampelt.
 * Daraus werden I- und Q-Signale fr die weitere Verarbeitung erzeugt.
 * Billige Variante:
 * I(n)=S(n)-S(n+2)
 * Q(n)=S(n+1)-S(n+3)
 *
 * I und Q werden dann tiefpassgefiltert und die Samplingfrequenz reduziert.
 * Anschlieend werden I und Q quadriert und addiert. 
 * Diese Summe wird wieder tiefpassgefiltert und nach einem Komparator die Pulsdauer gezhlt.
 * Alternativ knnte man statt des Komparators eine PLL einbauen und die Pulsdauer am Steuereingang des NCO abgreifen.
 *
 * ATmega32 und hher haben differentielle Analogeingnge mit einer programmierbaren Verstrkung von bis zu 200.
 * D.h. da sollte man eine auf abgestimmte Antenne direkt bzw. ber ein passives Bandfilter ranhngen knnen.
 * Wenn man den ADC-Noise Canceler verwendet, sind die Prozessorstrungen hoffentlich kein Problem. 
 * Die Bandbreite des AT-Eingangsteils liegt bei 4kHz. 
 */

#if PCTEST	
	#include "stdio.h"
	#include "debugpc.h"

	extern unsigned ADCH;
	extern unsigned ADCL;
	typedef long int32_t;
	typedef int int16_t;
	typedef unsigned long uint32_t;
	typedef unsigned uint16_t;
	typedef short unsigned uint8_t;
	
	void dout(int channel, long int val);	
#else
	#include <inttypes.h>
#endif
	volatile int32_t I_integral;
	volatile int32_t Q_integral;
	volatile uint8_t samplecntr=7;

void ifilter(int16_t I);
void qfilter(int16_t Q);
	
/**
 * called by ADC-Interrupt
 * should be moved to SIGNAL(ADC) directly, due to speed reasons
 *
 * 
 */
void dspint()
{
	static int16_t I;
	static int16_t Q;
	static uint8_t sampleptr;
	uint16_t sample = ADCL;
	sample |= ADCH<<8;
	
	sampleptr++;
	switch(sampleptr)
	{
		case 1:	I=sample; break;
		case 2:	Q=sample; 
			break;
		case 3:	I-=sample;
			ifilter(I);
			break;
		case 4: 
			Q-=sample;
			qfilter(Q); //Problem: wenn ich das alles in diesem Zeitschlitz mache, wird mir die Zeit vielleicht zu knapp! Ich sollte die Samples queuen und diese Sache im Main laufen lassen.
			sampleptr=0;
			break;
	}
}

/**
 * - low pass filtering of I-Signal
 * - decimation
 *
 * out=(2PI*fg/fs)*(I+last_sum*exp(-2PI*fg/fs))
 */
void ifilter(int16_t I)
{
	static int32_t sum;
			
	sum*=151;
	sum=sum>>8;	// => *0.59 => fg = ca. 445 Hz 
	
	sum+=I;
	
	dout(1,sum);
	
	//integral+=(sum*135)>>8;	//der absolute Wert ist hier wurscht. Es macht mir nichts aus, wenn das Filter eine Verstrkung hat.
	I_integral+=sum;
	
	dout(2,I_integral);
}
	
	
	
/**
 * - low pass filtering of Q-Signal
 * - decimation
 *
 * out=(2PI*fg/fs)*(I+last_sum*exp(-2PI*fg/fs))
 */
void qfilter(int16_t Q)
{
	static int32_t sum;
	

	sum*=151;
	sum=sum>>8;	// => *0.59 => fg = ca. 445 Hz 
	
	sum+=Q;
	
	dout(3,sum);
	
	//integral+=(sum*135)>>8;	//der absolute Wert ist hier wurscht. Es macht mir nichts aus, wenn das Filter eine Verstrkung hat.
	Q_integral+=sum;
	if (!samplecntr)
	{
		samplecntr=7;
		Q_integral=0;
		I_integral=0;
	}
	samplecntr--;
	dout(4,Q_integral);
}
	


/**
 * enables ADC and sets sampling rate
 */
void pulsdsp_init()
{
	dputs(__FUNCTION__ "\n");
	samplecntr=0;
	
}

/** 
 * reads decimated sampling data and processes it
 *
 * low pass filter frequency = 6 Hz
 */
void dsp_loop()
{

	uint32_t iqsum;
	int32_t i,q;
	uint32_t filtersum=0;
	dputs(__FUNCTION__ "\n");
	while(1)
	{	
		
		while(samplecntr)
			dsp_idle();
			;		//just wait, we can do other tasks here, inlcuding sleep?
			
		i=I_integral;//>>16;
		q=Q_integral;//>>16;
		i*=i;
		q*=q;
		q=q>>16;
		i=i>>16;
		iqsum=i+q;
		
		dout(5,iqsum);
		
		filtersum*=245;
		filtersum=filtersum>>8;
		filtersum+=iqsum;

		dout(6,filtersum);
		// Jetzt kommt der Komparator und die Zeitmessung

#if PCTEST
		samplecntr=7;
		Q_integral=0;
		I_integral=0;
#endif
	}

}

void pulsrxtest(void)
{
}

